@@ -23,6 +23,7 @@ import java.util.Stack;
 import java.util.logging.Logger;
 
 import net.morilib.lisp.CompiledCode.Oper;
+import net.morilib.lisp.chihaya.LispSyntax;
 import net.morilib.lisp.r6rs.LibraryID;
 import net.morilib.lisp.r6rs.LibraryIDException;
 import net.morilib.lisp.sos.LispClass;
@@ -451,7 +452,7 @@ import net.morilib.util.Stack2;
 
 
 	private void callSubr(
-			Subr sub, int addr,
+			ISubr sub, int addr,
 			Environment env, Datum d3d, Memento m,
 			ExcHandler hndl, Datum callee) {
 		m.addrStk.push(addr);
@@ -470,7 +471,8 @@ import net.morilib.util.Stack2;
 		m.finlStk.push(null);
 
 		try {
-			m.dataStk.push(sub.eval(d3d, env, m, message));
+			//m.dataStk.push(sub.eval(d3d, env, m, message));
+			m.dataStk.push(sub.eval(d3d, env, message));
 		} finally {
 			m.addrStk.pop();
 			m.codeStk.pop();
@@ -553,6 +555,10 @@ import net.morilib.util.Stack2;
 
 		// マクロ変換
 		d = comp.expandMacro(d, env, this, memento);
+		if(d instanceof LispSyntax) {
+			d = ((LispSyntax)d).getWrapped();
+		}
+//		d = MacroUtils72.unquote(d, new EnvironmentObject(env, false));
 		//System.out.println(LispUtils.getResult(d));
 
 		// コンパイル
@@ -667,6 +673,30 @@ import net.morilib.util.Stack2;
 		env.bindDatum(sym, mc);
 	}
 
+	private void mbind72(Environment env, Symbol sym, Datum dest) {
+		Macro72 mc;
+
+		if(dest instanceof Closure) {
+			mc = new Macro72((Closure)dest);
+			//((Closure)dest).setName(sym.getName());
+		} else {
+			throw message.getError("err.require.closure");
+		}
+		env.bindDatum(sym, mc);
+	}
+
+	private void mbindq(Environment env, Symbol sym, Datum dest) {
+		MacroQuote mc;
+
+		if(dest instanceof Closure) {
+			mc = new MacroQuote((Closure)dest);
+			//((Closure)dest).setName(sym.getName());
+		} else {
+			throw message.getError("err.require.closure");
+		}
+		env.bindDatum(sym, mc);
+	}
+
 	private Symbol getsym(Datum d) {
 		if(d instanceof SymbolName) {
 			return ((SymbolName)d).getSymbol();
@@ -844,6 +874,10 @@ import net.morilib.util.Stack2;
 					if(c.getDatum() instanceof ClosureClass) {
 						Closure ncl = new Closure(
 								(ClosureClass)c.getDatum(), env);
+//						Environment e2 = env.copyExceptRoot();
+//						Closure ncl = new Closure(
+//								(ClosureClass)c.getDatum(),
+//								e2);
 
 						//System.out.println(ncl.getParameterList());
 						m.dataStk.push(ncl);
@@ -896,6 +930,15 @@ import net.morilib.util.Stack2;
 					d14.appendAll(d13, message);
 					addr++;
 					break;
+				case APPEND_LIST_MULTI_VALUES:
+					Datum           d15 = m.dataStk.pop();
+					ConsListBuilder d16 = m.workStk.peek();
+
+					d16.appendAll(
+							LispUtils.listToCons(d15.getValues()),
+							message);
+					addr++;
+					break;
 				case END_LIST_DOT:
 					Datum           d21 = m.dataStk.pop();
 					//List<Datum>     d22 = m.workStk.pop();
@@ -929,9 +972,12 @@ import net.morilib.util.Stack2;
 
 					if(d3s instanceof Subr) {
 						Subr sub = (Subr)d3s;
-						ClosureClass cl = sub.getClosureClass(
-								env.getGlobal());
+						Environment ex2;
+						ClosureClass cl;
 
+						ex2 = (d3s instanceof ILispDynamicSubr) ?
+								env : env.getGlobal();
+						cl = sub.getClosureClass(ex2);
 						if(cl == null) {
 							// Subr is called with dynamic scope
 							callSubr(sub, addr, env, d3d, m, hndl, d3s);
@@ -939,7 +985,7 @@ import net.morilib.util.Stack2;
 						} else {
 							m.push(addr, code, env, hndl, d3s);
 
-							env = new Environment(env.getGlobal());
+							env = new Environment(ex2);
 							IntLispUtils.bindLocal(
 									cl.getParameterList(),
 									d3d, env, message);
@@ -947,6 +993,10 @@ import net.morilib.util.Stack2;
 							addr = 0;
 							//state = 0;
 						}
+					} else if(d3s instanceof ISubr) {
+						callSubr((ISubr)d3s, addr, env, d3d, m, hndl,
+								d3s);
+						addr++;
 					} else if(d3s instanceof Closure ||
 							d3s instanceof LispGeneric) {
 						Closure cl;
@@ -1160,14 +1210,16 @@ import net.morilib.util.Stack2;
 					addr = code.getAddress(c.getJmpLabel());
 					break;
 				case JMP_IF:
-					if(!LispBoolean.FALSE.equals(m.dataStk.peek())) {
+//					if(!LispBoolean.FALSE.equals(m.dataStk.peek())) {
+					if(m.dataStk.peek().isTrue()) {
 						addr = code.getAddress(c.getJmpLabel());
 					} else {
 						addr++;
 					}
 					break;
 				case JMP_UNLESS:
-					if(LispBoolean.FALSE.equals(m.dataStk.peek())) {
+//					if(LispBoolean.FALSE.equals(m.dataStk.peek())) {
+					if(!m.dataStk.peek().isTrue()) {
 						addr = code.getAddress(c.getJmpLabel());
 					} else {
 						addr++;
@@ -1192,6 +1244,9 @@ import net.morilib.util.Stack2;
 						//System.out.println("e2 :" + e2.toString());
 						//_log.finest("e2 :" + env.toString());
 						d51 = e2.findDatum(ss.getSymbol());
+						if(d51 == null) {
+							d51 = env.findDatum(ss);
+						}
 						//System.out.println(d51);
 					} else {
 						//_log.finest("env:" + env.toString());
@@ -1202,8 +1257,10 @@ import net.morilib.util.Stack2;
 						Datum zx = c.getDatum();
 
 						if(zx instanceof SymbolName &&
-								((SymbolName)zx).getName().equals("next-method")) {
-							d51 = env.findDatum(Symbol.getSymbol("next-method"));
+								((SymbolName)zx)
+								.getName().equals("next-method")) {
+							d51 = env.findDatum(
+									Symbol.getSymbol("next-method"));
 						}
 
 						if(d51 == null) {
@@ -1289,6 +1346,46 @@ import net.morilib.util.Stack2;
 					}
 					addr++;
 					break;
+				case BIND_MACRO72:
+					if(c.getDatum() instanceof SymbolScope) {
+						SymbolScope ss = (SymbolScope)c.getDatum();
+						Environment e2 = ss.getExecuteEnv(env);
+
+						mbind72(e2, ss.getSymbol(), m.dataStk.pop());
+					} else if(c.getDatum() instanceof Symbol) {
+						Symbol sym = (Symbol)c.getDatum();
+
+						if(tenv != null) {
+							mbind72(tenv, sym, m.dataStk.pop());
+						} else {
+							mbind72(env, sym, m.dataStk.pop());
+						}
+					} else {
+						throw message.getError(
+								"err.symbol", c.getDatum());
+					}
+					addr++;
+					break;
+				case BIND_MACRO_QUOTE:
+					if(c.getDatum() instanceof SymbolScope) {
+						SymbolScope ss = (SymbolScope)c.getDatum();
+						Environment e2 = ss.getExecuteEnv(env);
+
+						mbindq(e2, ss.getSymbol(), m.dataStk.pop());
+					} else if(c.getDatum() instanceof Symbol) {
+						Symbol sym = (Symbol)c.getDatum();
+
+						if(tenv != null) {
+							mbindq(tenv, sym, m.dataStk.pop());
+						} else {
+							mbindq(env, sym, m.dataStk.pop());
+						}
+					} else {
+						throw message.getError(
+								"err.symbol", c.getDatum());
+					}
+					addr++;
+					break;
 				case SET:
 					if(c.getDatum() instanceof SymbolScope) {
 						SymbolScope ss = (SymbolScope)c.getDatum();
@@ -1478,21 +1575,27 @@ import net.morilib.util.Stack2;
 
 					if(d84 instanceof InputPort) {
 						InputPort ipt = (InputPort)d84;
-						ClosureClass clc = makeLoad(ipt, env, m);
 
-						if(c.getOp().equals(Oper.LOAD_CODE)) {
-							m.push(addr, code, env, hndl, clc, 
-									Flags.LOAD);
-						} else {
-							m.push(addr, code, env, hndl, clc, 
-									Flags.LOAD_OV);
-							env = m.envStk.get(m.envStk.size() - 2);
+						try {
+							ClosureClass clc = makeLoad(ipt, env, m);
+
+							if(c.getOp().equals(Oper.LOAD_CODE)) {
+								m.push(addr, code, env, hndl, clc,
+										Flags.LOAD);
+							} else {
+								m.push(addr, code, env, hndl, clc,
+										Flags.LOAD_OV);
+								env = m.envStk.get(
+										m.envStk.size() - 2);
+							}
+							//env = new Environment(cl.getEnvironment());
+							code = clc.getCode();
+							addr = 0;
+							//state = 0;
+						} catch(LispException e) {
+							ipt.close();
+							throw e;
 						}
-
-						//env = new Environment(cl.getEnvironment());
-						code = clc.getCode();
-						addr = 0;
-						//state = 0;
 					} else {
 						throw message.getError("err.require.iport");
 					}
@@ -1501,18 +1604,54 @@ import net.morilib.util.Stack2;
 					Datum d85 = m.dataStk.pop();  // S exp
 					LispCompiler cp0 =
 						CompilerFactory.getInstance(message);
+					Datum rrr;
 
-					m.dataStk.push(cp0.expandMacro(
-							d85, env, this, new Memento()));
+					rrr = cp0.expandMacro(
+							d85, env, this, new Memento());
+//					rrr = MacroUtils72.unquote(rrr,
+//							new EnvironmentObject(env, false));
+					if(rrr instanceof LispSyntax) {
+						rrr = ((LispSyntax)rrr).getWrapped();
+					}
+					m.dataStk.push(rrr);
 					addr++;
 					break;
 				case EXPAND_MACRO1:
 					Datum d86 = m.dataStk.pop();  // S exp
 					LispCompiler cp1 =
 						CompilerFactory.getInstance(message);
+					Datum rrs;
 
-					m.dataStk.push(cp1.expandMacro1(
-							d86, env, this, new Memento()));
+					rrs = cp1.expandMacro1(
+							d86, env, this, new Memento());
+//					rrs = MacroUtils72.unquote(rrs,
+//							new EnvironmentObject(env, false));
+					if(rrs instanceof LispSyntax) {
+						rrs = ((LispSyntax)rrs).getWrapped();
+					}
+					m.dataStk.push(rrs);
+					addr++;
+					break;
+				case EXPAND_MACRO72:
+					Datum d861 = m.dataStk.pop();  // S exp
+					Datum d862 = m.dataStk.pop();  // S exp
+					LispCompiler cp2 =
+						CompilerFactory.getInstance(message);
+					Datum rrt;
+
+					if(!(d861 instanceof EnvironmentObject)) {
+						throw message.getError(
+								"err.environment", d861);
+					}
+					rrt = cp2.expandMacro(
+							d862,
+							((EnvironmentObject)d861).getEnvironment(),
+							this,
+							new Memento());
+					if(rrt instanceof LispSyntax) {
+						rrt = ((LispSyntax)rrt).getWrapped();
+					}
+					m.dataStk.push(rrt);
 					addr++;
 					break;
 				case FIND_LIST:
@@ -1858,19 +1997,19 @@ import net.morilib.util.Stack2;
 					addr = 0;
 					break;
 				case ENTER_EXCEPTION_HANDLER:
-//					hndl.lablStk.push(c.getJmpLabel());
+					hndl.lablStk.push(c.getJmpLabel());
 					hndl.hndlStk.push(m.dataStk.pop());
-//					hndl.codeStk.push(code);
+					hndl.codeStk.push(code);
 //					hndl.envStk.push(env);
-//					hndl.memStk.push(m.copy());
+					hndl.memStk.push(m.copy());
 					addr++;
 					break;
 				case LEAVE_EXCEPTION_HANDLER:
-//					hndl.lablStk.pop();
+					hndl.lablStk.pop();
 					hndl.hndlStk.pop();
-//					hndl.codeStk.pop();
+					hndl.codeStk.pop();
 //					hndl.envStk.pop();
-//					hndl.memStk.pop();
+					hndl.memStk.pop();
 					addr++;
 					break;
 				case GET_CURRENT_EXCEPTION_HANDLER:
@@ -1944,14 +2083,25 @@ import net.morilib.util.Stack2;
 //				}
 			} catch(LispException e) {
 				if(hndl.hndlStk.isEmpty()) {
-					throw e;
+					if(!(e instanceof SRFI34.RaisedException)) {
+						throw e;
+					} else if(((SRFI34.RaisedException)e).raised
+							instanceof ExceptionObject) {
+						throw ((ExceptionObject)
+								((SRFI34.RaisedException)e).raised)
+								.getException();
+					} else {
+						throw e;
+					}
 				} else {
 					Datum d3s = hndl.hndlStk.peek();
 					Datum d3d;
 //					code = hndl.codeStk.pop();
 //					env  = hndl.envStk.pop();
 //					addr = code.getAddress(hndl.lablStk.pop()) - 1;
-//					m.overwrite(hndl.memStk.pop());
+					m.overwrite(hndl.memStk.peek());
+					code = hndl.codeStk.peek();
+					addr = code.getAddress(hndl.lablStk.peek()) - 1;
 
 					if(e instanceof SRFI34.RaisedException) {
 						d3d = new Cons(
@@ -1983,6 +2133,10 @@ import net.morilib.util.Stack2;
 							addr = 0;
 							//state = 0;
 						}
+					} else if(d3s instanceof ISubr) {
+						callSubr((ISubr)d3s, addr, env, d3d, m, hndl,
+								d3s);
+						addr++;
 					} else if(d3s instanceof Closure ||
 							d3s instanceof LispGeneric) {
 						Closure cl;
@@ -2017,6 +2171,13 @@ import net.morilib.util.Stack2;
 									findmnext((LispGeneric)d3s, d3d));
 						}
 
+						// leave the handler
+						hndl.lablStk.pop();
+						hndl.hndlStk.pop();
+						hndl.codeStk.pop();
+//						hndl.envStk.pop();
+						hndl.memStk.pop();
+
 						code = cl.getCode();
 						addr = 0;
 						//state = 0;
