@@ -15,6 +15,8 @@
  */
 package net.morilib.lisp;
 
+import net.morilib.lisp.condition.LispCondition;
+
 public class LispArithmeticException extends LispException {
 
 	/**
@@ -32,8 +34,9 @@ public class LispArithmeticException extends LispException {
 	/**
 	 * @param message
 	 */
-	public LispArithmeticException(String code, String message) {
-		super(code, message);
+	public LispArithmeticException(String code, String message,
+			LispCondition condition) {
+		super(code, message, condition);
 	}
 
 }
