@@ -18,6 +18,9 @@ package net.morilib.lisp;
 import java.math.BigDecimal;
 import java.math.BigInteger;
 
+import net.morilib.lang.number.Rational;
+import net.morilib.lisp.math.constants.LispExp;
+import net.morilib.lisp.math.constants.LispSqrt;
 import net.morilib.lisp.subr.SubrUtils;
 import net.morilib.math.Math2;
 
@@ -30,10 +33,14 @@ import net.morilib.math.Math2;
 public final class LispMath {
 
 	//
-	private static final LispNumber TWO_I =
+	private static final LispComplex I =
+		LispComplex.newComplex(0.0, 1.0);
+	private static final LispComplex TWO_I =
 		LispComplex.newComplex(0.0, 2.0);
-	private static final LispNumber N_INF_PI_HALF_I =
-		LispComplex.newComplex(Double.NEGATIVE_INFINITY, -Math.PI / 2);
+	private static final LispComplex N_INF_PI_HALF_I =
+		LispComplex.newComplex(
+				Double.NEGATIVE_INFINITY,
+				-Math.PI / 2);
 	private static final Double P_ZERO = new Double( 0.0);
 	private static final Double M_ZERO = new Double(-0.0);
 	private static final BigInteger LIMIT_NRT =
@@ -60,7 +67,7 @@ public final class LispMath {
 	}
 
 	//
-	private static LispNumber exptC(LispNumber nm1, LispNumber nm2) {
+	private static LispComplex exptC(LispComplex nm1, LispComplex nm2) {
 		double a1 = nm1.getRealDouble();
 		double b1 = nm1.getImagDouble();
 		double r1 = Math.hypot(a1, b1);
@@ -99,7 +106,7 @@ public final class LispMath {
 	 * @param nm2
 	 * @return
 	 */
-	public static LispNumber expt(LispNumber nm1, LispNumber nm2) {
+	public static LispComplex expt(LispComplex nm1, LispComplex nm2) {
 		if(nm1.isNaN() || nm2.isNaN()) {
 			return LispDouble.NaN;
 		} else if(nm2.isZero()) {
@@ -216,7 +223,7 @@ public final class LispMath {
 	 * @param r
 	 * @return
 	 */
-	public static LispNumber expt(LispNumber nm1, double r) {
+	public static LispComplex expt(LispComplex nm1, double r) {
 		double a1 = nm1.getRealDouble();
 		double b1 = nm1.getImagDouble();
 
@@ -264,14 +271,48 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber sqrt(LispNumber n) {
-		BigInteger b;
+	public static LispComplex sqrt(LispComplex n) {
+		BigInteger b, c;
+		Rational r;
+		LispReal x;
 
-		if(n.isExact() && n.isInteger() &&
-				((LispReal)n).signum() > 0 &&
-				(b = Math2.sqrtExact(
-						n.getBigInteger())).signum() > 0) {
-			return LispInteger.valueOf(b);
+		if(n.isZero()) {
+			return LispInteger.ZERO;
+		} else if(n.isOne()) {
+			return LispInteger.ONE;
+		} else if(n.isExact() && n.isInteger()) {
+			b = Math2.sqrtExact(n.getBigInteger().abs());
+			if(b.signum() > 0) {
+				x = LispInteger.valueOf(b);
+			} else {
+				x = (LispReal)LispSqrt.valueOf(
+						n.getBigInteger().abs(),
+						Rational.ONE, Rational.ZERO);
+			}
+
+			if(n.getBigInteger().signum() > 0) {
+				return x;
+			} else {
+				return LispComplex.newComplex(LispInteger.ZERO, x);
+			}
+		} else if(n.isExact() && n.isRational()) {
+			r = ((LispExactReal)n).getRational();
+			b = Math2.sqrtExact(r.getNumerator().toBigInteger().abs());
+			c = Math2.sqrtExact(
+					r.getDenominator().toBigInteger().abs());
+			if(b.signum() > 0 && c.signum() > 0) {
+				x = LispRational.newRational(b, c);
+			} else {
+				x = (LispReal)LispSqrt.valueOf(
+						(r.signum() > 0 ? r : r.negate()),
+						Rational.ONE, Rational.ZERO);
+			}
+
+			if(r.signum() > 0) {
+				return x;
+			} else {
+				return LispComplex.newComplex(LispInteger.ZERO, x);
+			}
 		} else if(n.isReal()) {
 			if(n.getRealDouble() == 0.0) {
 				return new LispDouble(0.0);
@@ -291,8 +332,11 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber exp(LispNumber n) {
-		if(n.isReal()) {
+	public static LispComplex exp(LispComplex n) {
+		if(n.isRational() && n.isExact()) {
+			return LispExp.valueOf(((LispExactReal)n).toRational(),
+					Rational.ONE, Rational.ZERO);
+		} else if(n.isReal()) {
 			return new LispDouble(Math.exp(n.getRealDouble()));
 		} else {
 			double a = n.getRealDouble();
@@ -309,8 +353,14 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber log(LispNumber n) {
-		if(n.getReal().isZero() && isMinusZero(n.getImagDouble())) {
+	public static LispComplex log(LispComplex n) {
+		LispExp e;
+
+		if(n instanceof LispExp &&
+				(e = (LispExp)n).getScale().isUnit()) {
+			return LispRational.valueOf(e.getX());
+		} else if(n.getReal().isZero() &&
+				isMinusZero(n.getImagDouble())) {
 			return N_INF_PI_HALF_I;
 		} else if(n.isReal() && n.getReal().signum() > 0) {
 			return new LispDouble(Math.log(n.getRealDouble()));
@@ -329,7 +379,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber sin(LispNumber n) {
+	public static LispComplex sin(LispComplex n) {
 		if(n.isReal()) {
 			return new LispDouble(Math.sin(n.getRealDouble()));
 		} else if(n.getReal().isZero()) {
@@ -352,7 +402,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber cos(LispNumber n) {
+	public static LispComplex cos(LispComplex n) {
 		if(n.isReal()) {
 			return new LispDouble(Math.cos(n.getRealDouble()));
 		} else if(n.getReal().isZero()) {
@@ -375,7 +425,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber tan(LispNumber n) {
+	public static LispComplex tan(LispComplex n) {
 		if(n.isReal()) {
 			return new LispDouble(Math.tan(n.getRealDouble()));
 		} else if(n.getReal().isZero()) {
@@ -384,20 +434,20 @@ public final class LispMath {
 			return LispComplex.newComplex(0.0, Math.tanh(y));
 		} else if(n.getImagDouble() == Double.POSITIVE_INFINITY) {
 			double x = n.getRealDouble();
-			LispNumber nm, dn;
+			LispComplex nm, dn;
 
 			nm = LispComplex.newComplex(Math.tan(x), 1);
 			dn = LispComplex.newComplex(1, Math.tan(x));
-			return nm.div(dn);
+			return (LispComplex)nm.div(dn);
 		} else if(n.getImagDouble() == Double.NEGATIVE_INFINITY) {
 			double x = n.getRealDouble();
-			LispNumber nm, dn;
+			LispComplex nm, dn;
 
 			nm = LispComplex.newComplex(Math.tan(x), -1);
 			dn = LispComplex.newComplex(1, -Math.tan(x));
-			return nm.div(dn);
+			return (LispComplex)nm.div(dn);
 		} else {
-			return sin(n).div(cos(n));
+			return (LispComplex)sin(n).div(cos(n));
 		}
 	}
 
@@ -406,7 +456,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber sinDeg(LispNumber n) {
+	public static LispComplex sinDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			int b, s;
 
@@ -432,7 +482,7 @@ public final class LispMath {
 				return LispInteger.valueOf(s);
 			}
 		}
-		return sin(n.mul(INEXACT_RAD_PER_DEG));
+		return sin((LispComplex)n.mul(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -440,7 +490,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber cosDeg(LispNumber n) {
+	public static LispComplex cosDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			int b, s;
 
@@ -466,7 +516,7 @@ public final class LispMath {
 				return LispInteger.ZERO;
 			}
 		}
-		return cos(n.mul(INEXACT_RAD_PER_DEG));
+		return cos((LispComplex)n.mul(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -474,7 +524,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber tanDeg(LispNumber n) {
+	public static LispComplex tanDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			int b, s;
 
@@ -500,7 +550,7 @@ public final class LispMath {
 				break;
 			}
 		}
-		return tan(n.mul(INEXACT_RAD_PER_DEG));
+		return tan((LispComplex)n.mul(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -508,7 +558,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber asin(LispNumber n) {
+	public static LispComplex asin(LispComplex n) {
 		double x = n.getRealDouble();
 		double y = n.getImagDouble();
 
@@ -531,9 +581,10 @@ public final class LispMath {
 		} else if(y == Double.NEGATIVE_INFINITY) {
 			return LispComplex.newComplex(0, y);
 		} else {
-			LispNumber z1 = LispComplex.I.mul(n);
-			LispNumber z2 = sqrt(LispDouble.ONE.sub(n.mul(n)));
-			LispNumber z3 = log(z1.add(z2));
+			LispComplex z1 = (LispComplex)I.mul(n);
+			LispComplex z2 = sqrt((LispComplex)LispDouble.ONE.sub(
+					n.mul(n)));
+			LispComplex z3 = log((LispComplex)z1.add(z2));
 
 			//return LispComplex.MINUS_I.mul(z3);
 			return LispComplex.newComplex(
@@ -546,15 +597,15 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber acos(LispNumber n) {
+	public static LispComplex acos(LispComplex n) {
 		double x = n.getRealDouble();
 
 		if(n.isReal() && x >= -1.0 && x <= 1.0) {
 			return new LispDouble(Math.acos(n.getRealDouble()));
 		} else {
-			LispNumber pi2 = new LispDouble(Math.PI / 2);
+			LispComplex pi2 = new LispDouble(Math.PI / 2);
 
-			return pi2.sub(asin(n));
+			return (LispComplex)pi2.sub(asin(n));
 		}
 	}
 
@@ -563,7 +614,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber atan(LispNumber n) {
+	public static LispComplex atan(LispComplex n) {
 		if(n.isReal()) {
 			return new LispDouble(Math.atan(n.getRealDouble()));
 		} else if(Double.isInfinite(n.getImagDouble())) {
@@ -576,12 +627,20 @@ public final class LispMath {
 			} else {
 				return new LispDouble(-Math.PI / 2);
 			}
+		} else if(n.getRealDouble() == 0.0 &&
+				n.getImagDouble() == 1.0) {
+			return LispComplex.newComplex(0, Double.POSITIVE_INFINITY);
+		} else if(n.getRealDouble() == 0.0 &&
+				n.getImagDouble() == -1.0) {
+			return LispComplex.newComplex(0, Double.NEGATIVE_INFINITY);
 		} else {
-			LispNumber z1 = LispDouble.ONE.add(LispComplex.I.mul(n));
-			LispNumber z2 = LispDouble.ONE.sub(LispComplex.I.mul(n));
-			LispNumber z3 = log(z1).sub(log(z2));
+			LispComplex z1 = (LispComplex)LispDouble.ONE.add(
+					I.mul(n));
+			LispComplex z2 = (LispComplex)LispDouble.ONE.sub(
+					I.mul(n));
+			LispComplex z3 = (LispComplex)log(z1).sub(log(z2));
 
-			return z3.div(TWO_I);
+			return (LispComplex)z3.div(TWO_I);
 		}
 	}
 
@@ -590,7 +649,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber asinDeg(LispNumber n) {
+	public static LispComplex asinDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			Integer x = SubrUtils.toIntExact(n.getBigInteger());
 
@@ -615,7 +674,7 @@ public final class LispMath {
 				return LispInteger.valueOf(-30);
 			}
 		}
-		return asin(n.div(INEXACT_RAD_PER_DEG));
+		return asin((LispComplex)n.div(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -623,7 +682,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber acosDeg(LispNumber n) {
+	public static LispComplex acosDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			Integer x = SubrUtils.toIntExact(n.getBigInteger());
 
@@ -648,7 +707,7 @@ public final class LispMath {
 				return LispInteger.valueOf(120);
 			}
 		}
-		return acos(n.div(INEXACT_RAD_PER_DEG));
+		return acos((LispComplex)n.div(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -656,7 +715,7 @@ public final class LispMath {
 	 * @param n
 	 * @return
 	 */
-	public static LispNumber atanDeg(LispNumber n) {
+	public static LispComplex atanDeg(LispComplex n) {
 		if(n.isExact() && n.isInteger()) {
 			Integer x = SubrUtils.toIntExact(n.getBigInteger());
 
@@ -670,7 +729,7 @@ public final class LispMath {
 				return LispInteger.valueOf(-45);
 			}
 		}
-		return atan(n.div(INEXACT_RAD_PER_DEG));
+		return atan((LispComplex)n.div(INEXACT_RAD_PER_DEG));
 	}
 
 	/**
@@ -766,4 +825,37 @@ public final class LispMath {
 		}
 	}
 
+	/**
+	 * 
+	 * @param a
+	 * @param b
+	 * @param c
+	 * @return
+	 */
+	public static LispComplex[] solveQuadratic(LispReal a, LispReal b,
+			LispReal c) {
+		LispReal D = b.multiply(b).subtract(a.multiply(c).multiply(4));
+		LispReal dr, db, da = a.multiply(2);
+		int sd;
+
+		if((sd = D.signum()) > 0) {
+			dr = sqrt(D).getReal();
+			return new LispComplex[] {
+					b.negate().add(dr).divide(da),
+					b.negate().subtract(dr).divide(da)
+			};
+		} else if(sd == 0) {
+			return new LispComplex[] {
+					b.negate().divide(da)
+			};
+		} else {
+			db = b.negate().divide(da);
+			dr = sqrt(D.negate()).getReal().divide(da);
+			return new LispComplex[] {
+					LispComplex.newComplex(db, dr),
+					LispComplex.newComplex(db, dr.negate())
+			};
+		}
+	}
+
 }
