@@ -23,6 +23,11 @@ import java.util.Map;
 import java.util.MissingResourceException;
 import java.util.ResourceBundle;
 
+import net.morilib.lisp.condition.LispCompoundCondition;
+import net.morilib.lisp.condition.LispCondition;
+import net.morilib.lisp.condition.LispSimpleCondition;
+import net.morilib.util.Maps;
+
 /**
  * 
  *
@@ -46,10 +51,12 @@ public final class LispMessage {
 	private ResourceBundle messages;
 	private List<ResourceBundle> pluggable =
 		new ArrayList<ResourceBundle>();
+	private Locale locale;
 
 	//
 	private LispMessage(Locale lc) {
 		messages = ResourceBundle.getBundle(CLSNAME, lc);
+		locale   = lc;
 	}
 
 	/**
@@ -146,13 +153,65 @@ public final class LispMessage {
 		return get(prop) + ":" + LispUtils.getResult(d);
 	}
 
+	/**
+	 * 
+	 * @param prop
+	 * @return
+	 */
+	public static LispCondition getConditionByKey(String prop) {
+		LispSimpleCondition err;
+
+		if(Maps.matchPropertyKey(
+				"err.require.**", prop) != null) {
+			err = LispSimpleCondition.newInstance("&assertion");
+		} else if(Maps.matchPropertyKey(
+				"err.*.require.**", prop) != null) {
+			err = LispSimpleCondition.newInstance("&assertion");
+		} else if(prop.equals("err.argument")) {
+			err = LispSimpleCondition.newInstance("&assertion");
+		} else if(Maps.matchPropertyKey(
+				"err.read.**", prop) != null) {
+			err = LispSimpleCondition.newInstance("&lexical");
+		} else if(Maps.matchPropertyKey(
+				"err.io.**", prop) != null) {
+			err = LispSimpleCondition.newInstance("&error");
+		} else if(Maps.matchPropertyKey(
+				"err.jdbc.**", prop) != null) {
+			err = LispSimpleCondition.newInstance("&error");
+		} else if(prop.equals("err.unbound")) {
+			err = LispSimpleCondition.newInstance("&undefined");
+		} else if(prop.equals("err.srfi34.raised")) {
+			err = LispSimpleCondition.newInstance("&non-continuable");
+		} else if(prop.equals("err.user")) {
+			err = LispSimpleCondition.newInstance("&non-continuable");
+		} else {
+			err = LispSimpleCondition.newInstance("&violation");
+		}
+		return err;
+	}
+
+	/**
+	 * 
+	 * @param prop
+	 * @return
+	 */
+	public LispCondition getCondition(String prop) {
+		LispCondition err;
+		LispSimpleCondition msg;
+
+		err = getConditionByKey(prop);
+		msg = LispSimpleCondition.newInstance("&message");
+		msg.setField("message", new LispString(get(prop)));
+		return new LispCompoundCondition(err, msg);
+	}
+
 	/**
 	 * 
 	 * @param prop
 	 * @return
 	 */
 	public LispException getError(String prop) {
-		return new LispException(prop, get(prop));
+		return new LispException(prop, get(prop), getCondition(prop));
 	}
 
 	/**
@@ -162,7 +221,8 @@ public final class LispMessage {
 	 * @return
 	 */
 	public LispException getError(String prop, String msg2) {
-		return new LispException(prop, get(prop, msg2));
+		return new LispException(
+				prop, get(prop, msg2), getCondition(prop));
 	}
 
 	/**
@@ -172,7 +232,8 @@ public final class LispMessage {
 	 * @return
 	 */
 	public LispException getError(String prop, Datum d) {
-		return new LispException(prop, get(prop, d));
+		return new LispException(prop, get(prop, d),
+				getCondition(prop));
 	}
 
 	/**
@@ -181,7 +242,8 @@ public final class LispMessage {
 	 * @return
 	 */
 	public LispException getError(String prop, Throwable th) {
-		return new LispException(prop, get(prop), th);
+		return new LispException(prop, get(prop), getCondition(prop),
+				th);
 	}
 
 	/**
@@ -192,7 +254,8 @@ public final class LispMessage {
 	 */
 	public LispException getError(
 			String prop, String msg2, Throwable th) {
-		return new LispException(prop, get(prop, msg2), th);
+		return new LispException(
+				prop, get(prop, msg2), getCondition(prop), th);
 	}
 
 	/**
@@ -203,7 +266,8 @@ public final class LispMessage {
 	 */
 	public LispException getError(
 			String prop, Datum d, Throwable th) {
-		return new LispException(prop, get(prop, d), th);
+		return new LispException(
+				prop, get(prop, d), getCondition(prop), th);
 	}
 
 	/**
@@ -212,7 +276,7 @@ public final class LispMessage {
 	 * @return
 	 */
 	public ReadException getReadError(String prop) {
-		return new ReadException(prop, get(prop));
+		return new ReadException(prop, get(prop), getCondition(prop));
 	}
 
 	/**
@@ -222,7 +286,8 @@ public final class LispMessage {
 	 * @return
 	 */
 	public ReadException getReadError(String prop, String msg2) {
-		return new ReadException(prop, get(prop, msg2));
+		return new ReadException(prop, get(prop, msg2),
+				getCondition(prop));
 	}
 
 	/**
@@ -232,7 +297,8 @@ public final class LispMessage {
 	 * @return
 	 */
 	public ReadException getReadError(String prop, Datum d) {
-		return new ReadException(prop, get(prop, d));
+		return new ReadException(prop, get(prop, d),
+				getCondition(prop));
 	}
 
 	/**
@@ -242,7 +308,25 @@ public final class LispMessage {
 	public LispException getUncaughtException(LispException e) {
 		return new LispException(
 				"err.srfi18.uncaught",
-				get("err.srfi18.uncaught"), e);
+				get("err.srfi18.uncaught"),
+				getCondition("err.srfi18.uncaught"), e);
+	}
+
+	/**
+	 * 
+	 * @param prop
+	 */
+	public void warn(String prop) {
+		System.err.print(get("warn.repl.err"));
+		System.err.print(" ");
+		System.err.println(get(prop));
+	}
+
+	/**
+	 * @return the locale
+	 */
+	public Locale getLocale() {
+		return locale;
 	}
 
 }
