@@ -21,15 +21,18 @@ import java.io.IOException;
 import java.io.InputStream;
 import java.io.InputStreamReader;
 import java.io.OutputStream;
+import java.io.PrintWriter;
 import java.io.PushbackReader;
 import java.io.Reader;
 import java.math.BigInteger;
+import java.util.ArrayList;
 import java.util.HashSet;
 import java.util.LinkedList;
 import java.util.List;
 import java.util.Locale;
 import java.util.logging.Logger;
 
+import net.morilib.lisp.exlib.CommandLine;
 import net.morilib.lisp.r6rs.LibraryID;
 import net.morilib.lisp.util.LogEnv;
 import net.morilib.parser.ParserUtils;
@@ -77,6 +80,9 @@ public final class Scheme {
 				Maps.identityMapOf(new HashSet<Symbol>(sym)));
 	}
 
+	//
+	private static PrintWriter transcript = null;
+
 	//
 	private Environment  global;
 	private LispCompiler comp;
@@ -178,6 +184,15 @@ public final class Scheme {
 		return newRnRS(ver).global;
 	}
 
+	/**
+	 * 
+	 * @param ver
+	 * @return
+	 */
+	public static Scheme newEmpty() {
+		return new Scheme(LispMessage.getInstance());
+	}
+
 	/**
 	 * 
 	 * @param ver
@@ -210,6 +225,45 @@ public final class Scheme {
 		return res;
 	}
 
+	/**
+	 * 
+	 * @param pw
+	 */
+	public static void setTranscript(PrintWriter pw) {
+		synchronized(Scheme.class) {
+			if(transcript != null) {
+				transcript.close();
+			}
+			transcript = pw;
+		}
+	}
+
+	/**
+	 * 
+	 * @param ver
+	 * @return
+	 */
+	public void loadNull(int ver) {
+		InitSubrLoader.loadNullEnv(global, ver);
+		InitVarLoader.loadNullEnv(global, ver);
+		parser.clear();
+		InitLispLoader.loadNullEnv(this, ver);
+		parser.clear();
+	}
+
+	/**
+	 * 
+	 * @param ver
+	 * @return
+	 */
+	public void loadRnRS(int ver) {
+		InitSubrLoader.loadRnRSEnv(global, ver);
+		InitVarLoader.loadRnRSEnv(global, ver);
+		parser.clear();
+		InitLispLoader.loadRnRSEnv(this, ver);
+		parser.clear();
+	}
+
 	/**
 	 * 
 	 * @param sexp
@@ -221,13 +275,13 @@ public final class Scheme {
 			CompiledCode.Builder b = new CompiledCode.Builder();
 			long t;
 
-			// マクロ変換
+			// macro extension
 			t = System.currentTimeMillis();
 			d = comp.expandMacro(d, global, exec, memento);
 			//System.out.println(LispUtils.getResult(d));
 			IntLispUtils.timelog(_log, "Expand Macro : ", t);
 
-			// コンパイル
+			// compile
 			t = System.currentTimeMillis();
 			comp.compile(
 					d, global, b, true, new Cons(), true,
@@ -236,13 +290,13 @@ public final class Scheme {
 			b.addReturnOp();
 			IntLispUtils.timelog(_log, "Compile : ", t);
 
-			// 実行
+			// execute
 			//System.out.println(b.getCodeRef().toString());
 			t = System.currentTimeMillis();
 			Datum res = exec.exec(b.getCodeRef(), global, memento);
 			IntLispUtils.timelog(_log, "Execute : ", t);
 
-			// スタックの表示
+			// display the stack
 			_log.fine(memento.toString());
 
 			//parser.clear();
@@ -347,6 +401,7 @@ public final class Scheme {
 		while(true) {
 			String prompt = " >";
 			Datum res = null;
+			String rep;
 
 			try {
 				while(res == null) {
@@ -355,20 +410,37 @@ public final class Scheme {
 
 					if(r2 == null) {
 						break reploop;
+					} else if(transcript != null) {
+						synchronized(Scheme.class) {
+							if(transcript != null) {
+								transcript.print(prompt);
+								transcript.println(r2);
+							}
+						}
 					}
 					res = input(r2);
 					prompt = ">>";
 				}
 
-				// 結果表示
-				System.out.println(LispUtils.getResult(res));
+				// print the result
+				rep = LispUtils.getResult(res);
+				System.out.println(rep);
+				if(transcript != null) {
+					synchronized(Scheme.class) {
+						if(transcript != null) {
+							transcript.println(rep);
+						}
+					}
+				}
 
-				// スタックの表示
+				// print the stack
 				//System.out.println(memento);
 			} catch(ReadException e) {
 				System.err.println(
 						message.get("err.repl.read") +
 						e.getMessage());
+				System.err.println(message.get("info.condition") +
+						e.getCondition().describeShort());
 				//e.printStackTrace();
 				//parser.clear();
 				//memento = exec.newMemento();
@@ -385,6 +457,8 @@ public final class Scheme {
 				if(e.getException() != null) {
 					e.getException().printStackTrace();
 				}
+				System.err.println(message.get("info.condition") +
+						e.getCondition().describeShort());
 				_log.fine("Stack trace\n" + tra);
 				//e.printStackTrace();
 				//parser.clear();
@@ -407,6 +481,13 @@ public final class Scheme {
 	 */
 	public static void main(String[] args) throws IOException {
 		Scheme eval = Scheme.newInstance();
+		List<Datum> d1 = new ArrayList<Datum>();
+
+		d1.add(new LispString("schluessel"));
+		for(String s : args) {
+			d1.add(new LispString(s));
+		}
+		CommandLine.commandLine = LispUtils.listToCons(d1);
 
 		if(args.length == 0) {
 			eval.readEvalPrintLoop(new InputStreamReader(System.in));
@@ -617,6 +698,8 @@ public final class Scheme {
 			if(((Subr)proc).createClosureClass(env) == null) {
 				return ((Subr)proc).eval(lst, env, mesg);
 			}
+		} else if(proc instanceof ISubr) {
+			return ((ISubr)proc).eval(lst, env, mesg);
 		}
 
 		//
@@ -844,4 +927,11 @@ public final class Scheme {
 		return exec.exec(cd.getCodeRef(), global, memento);
 	}
 
+	/**
+	 * @return the message
+	 */
+	public LispMessage getMessage() {
+		return message;
+	}
+
 }
