@@ -19,9 +19,20 @@ import java.util.List;
 
 import net.morilib.lisp.LispCompiler.MiscInfo;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2009
+ */
 public class SynBegin extends Syntax {
-	
-	
+
+	/**
+	 * 
+	 */
+	public static final Symbol BEGIN = Symbol.getSymbol("begin");
+
+	//
 	/*package*/ void compile(
 			Datum body,
 			Environment env,
@@ -37,10 +48,10 @@ public class SynBegin extends Syntax {
 			MiscInfo syncased) {
 		SyntaxUtils.compileList(
 				body, env, comp, build, callsym, istail, mesg, symlist,
-				exec, memento, syncased);
+				exec, memento, toplevel, syncased);
 	}
-	
-	
+
+	//
 	/*package*/ Datum replaceLocalVals(
 			Datum body,
 			Environment env,
@@ -52,5 +63,5 @@ public class SynBegin extends Syntax {
 		return SyntaxUtils.replaceLocalValsList(
 				body, env, comp, ienv, mesg, toplv, ttype);
 	}
-	
+
 }
