@@ -761,12 +761,36 @@ public class LispHash extends Datum2 implements LispMap {
 
 	//
 	private static final Map<Datum, Datum> DEFHASH;
-	private static final Procedure EQ      = new IsEq();
-	private static final Procedure EQV     = new IsEqv();
-	private static final Procedure EQUAL   = new IsEqual();
-	private static final Procedure EQ_H    = new HashByIdentity();
-	private static final Procedure EQV_H   = new Hasv();
-	private static final Procedure EQUAL_H = new Hash();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQ         = new IsEq();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQV        = new IsEqv();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQUAL      = new IsEqual();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQ_HASH    = new HashByIdentity();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQV_HASH   = new Hasv();
+
+	/**
+	 * 
+	 */
+	public static final Procedure EQUAL_HASH = new Hash();
 
 	static {
 		DEFHASH = new HashMap<Datum, Datum>();
@@ -774,9 +798,9 @@ public class LispHash extends Datum2 implements LispMap {
 		DEFHASH.put(new StringCIEqual(), new StringCiHash());
 		DEFHASH.put(new CharEqual(),     new CharHash());
 		DEFHASH.put(new CharCIEqual(),   new CharCiHash());
-		DEFHASH.put((Datum)EQ,           (Datum)EQ_H);
-		DEFHASH.put((Datum)EQV,          (Datum)EQV_H);
-		DEFHASH.put((Datum)EQUAL,        (Datum)EQUAL_H);
+		DEFHASH.put((Datum)EQ,           (Datum)EQ_HASH);
+		DEFHASH.put((Datum)EQV,          (Datum)EQV_HASH);
+		DEFHASH.put((Datum)EQUAL,        (Datum)EQUAL_HASH);
 	}
 
 	//
@@ -797,6 +821,17 @@ public class LispHash extends Datum2 implements LispMap {
 		this.message   = message;
 	}
 
+	/**
+	 * 
+	 */
+	public LispHash() {
+		this.hashmap   = new HashMap<Object, Datum>();
+		this.eqproc    = EQV;
+		this.hashproc  = EQV_HASH;
+		this.activeenv = new Environment();
+		this.message   = LispMessage.getInstance();
+	}
+
 	/**
 	 * @param lispHash
 	 */
@@ -816,13 +851,13 @@ public class LispHash extends Datum2 implements LispMap {
 		} else if(p1.equals(EQ)) {
 			return new LispHash(
 					new IdentityHashMap<Object, Datum>(),
-					EQ, EQ_H, env, mesg);
+					EQ, EQ_HASH, env, mesg);
 		} else if(p2 != null || (p2 = DEFHASH.get(p1)) != null) {
 			return new LispHash(new HashMap<Object, Datum>(),
 					(Procedure)p1, (Procedure)p2, env, mesg);
 		} else {
 			return new LispHash(new HashMap<Object, Datum>(),
-					EQUAL, EQUAL_H, env, mesg);
+					EQUAL, EQUAL_HASH, env, mesg);
 		}
 	}
 
@@ -974,15 +1009,16 @@ public class LispHash extends Datum2 implements LispMap {
 	 * @see net.morilib.lisp.collection.LispCollection#equivalence(net.morilib.lisp.Datum, net.morilib.lisp.Datum)
 	 */
 	public boolean equivalence(Datum a, Datum b) {
-		if(a instanceof Cons && b instanceof Cons) {
-			Cons x = (Cons)a;
-			Cons y = (Cons)b;
-
-			return (equivalenceKey  (x.getCar(), y.getCar()) &&
-					equivalenceValue(x.getCdr(), y.getCdr()));
-		} else {
-			return equivalenceKey(a, b);
-		}
+//		if(a instanceof Cons && b instanceof Cons) {
+//			Cons x = (Cons)a;
+//			Cons y = (Cons)b;
+//
+//			return (equivalenceKey  (x.getCar(), y.getCar()) &&
+//					equivalenceValue(x.getCdr(), y.getCdr()));
+//		} else {
+//			return equivalenceKey(a, b);
+//		}
+		return LispCollections.equivalence(this, a, b);
 	}
 
 	/* (non-Javadoc)
@@ -1008,22 +1044,23 @@ public class LispHash extends Datum2 implements LispMap {
 		if(col instanceof LispHash) {
 			return hashmap.equals(((LispHash)col).hashmap);
 		} else {
-			for(Datum x : col) {
-				if(x instanceof Cons) {
-					if(!contains(x)) {
-						return false;
-					}
-				} else {
-					return false;
-				}
-			}
-
-			for(Datum e : this) {
-				if(!col.contains(e)) {
-					return false;
-				}
-			}
-			return true;
+//			for(Datum x : col) {
+//				if(x instanceof Cons) {
+//					if(!contains(x)) {
+//						return false;
+//					}
+//				} else {
+//					return false;
+//				}
+//			}
+//
+//			for(Datum e : this) {
+//				if(!col.contains(e)) {
+//					return false;
+//				}
+//			}
+//			return true;
+			return LispCollections.equalsMapEntry(this, col);
 		}
 	}
 
