@@ -15,17 +15,29 @@
  */
 package net.morilib.lisp.exlib;
 
+import net.morilib.lisp.ConsIterator;
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
 import net.morilib.lisp.LispMessage;
 import net.morilib.lisp.Nil;
 import net.morilib.lisp.Subr;
+import net.morilib.lisp.subr.SubrUtils;
+import net.morilib.util.Iterators;
 
 public class Exit extends Subr {
 
 	@Override
 	public Datum eval(Datum body, Environment env, LispMessage mesg) {
-		System.exit(0);
+		ConsIterator itr = new ConsIterator(body);
+		Datum d1 = Iterators.nextIf(itr);
+
+		if(d1 == null) {
+			System.exit(0);
+		} else if(d1.isTrue()) {
+			System.exit(SubrUtils.getU8(d1, mesg));
+		} else {
+			System.exit(72);
+		}
 		return Nil.NIL;
 	}
 
