@@ -30,11 +30,13 @@ public class CharCIEqual extends CharCompare {
 	 */
 	@Override
 	protected boolean compare(LispCharacter c1a, LispCharacter c2a) {
-		char ch1 = Character.toUpperCase(c1a.getCharacter());
-		char ch2 = Character.toUpperCase(c2a.getCharacter());
+//		char ch1 = Character.toUpperCase(c1a.getCharacter());
+//		char ch2 = Character.toUpperCase(c2a.getCharacter());
+		int ch1 = SubrUtils.toFoldCase(c1a.getCharacterCodePoint());
+		int ch2 = SubrUtils.toFoldCase(c2a.getCharacterCodePoint());
 
-		ch1 = Character.toLowerCase(ch1);
-		ch2 = Character.toLowerCase(ch2);
+//		ch1 = Character.toLowerCase(ch1);
+//		ch2 = Character.toLowerCase(ch2);
 		return ch1 == ch2;
 	}
 
