@@ -17,19 +17,31 @@ package net.morilib.lisp.subr;
 
 import net.morilib.lisp.Datum;
 import net.morilib.lisp.Environment;
+import net.morilib.lisp.LispComplex;
 import net.morilib.lisp.LispMath;
 import net.morilib.lisp.LispMessage;
-import net.morilib.lisp.LispNumber;
+import net.morilib.lisp.math.angle.ILispAngle;
 
+/**
+ * 
+ *
+ *
+ * @author MORIGUCHI, Yuichiro 2009
+ */
 public class Cos extends UnaryArgs {
 
+	/* (non-Javadoc)
+	 * @see net.morilib.lisp.subr.UnaryArgs#execute(net.morilib.lisp.Datum, net.morilib.lisp.Environment, net.morilib.lisp.LispMessage)
+	 */
 	@Override
-	protected Datum execute(Datum c1a, Environment env, LispMessage mesg) {
-		if(c1a instanceof LispNumber) {
-			return LispMath.cos((LispNumber)c1a);
+	protected Datum execute(Datum c1a, Environment env,
+			LispMessage mesg) {
+		if(c1a instanceof LispComplex) {
+			return LispMath.cos((LispComplex)c1a);
+		} else if(c1a instanceof ILispAngle) {
+			return ((ILispAngle)c1a).cos();
 		} else {
-			throw mesg.getError("err.require.number", c1a);
-			//throw new LispException("number required");
+			throw mesg.getError("err.require.complex", c1a);
 		}
 	}
 
