@@ -27,6 +27,19 @@ import net.morilib.lisp.Subr;
  */
 public abstract class NoArgs extends Subr {
 
+	/**
+	 * 
+	 */
+	protected NoArgs() { }
+
+	/**
+	 * 
+	 * @param s
+	 */
+	protected NoArgs(String s) {
+		super(s);
+	}
+
 	/**
 	 * 
 	 * @param env
